package dev.kske.chess.game;

import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.List;
import java.util.stream.Collectors;

import dev.kske.chess.board.Board;
import dev.kske.chess.board.Move;
import dev.kske.chess.board.Move.Type;
import dev.kske.chess.board.Piece.Color;
import dev.kske.chess.board.Position;
import dev.kske.chess.ui.OverlayComponent;

/**
 * Project: <strong>Chess</strong><br>
 * File: <strong>NaturalPlayer.java</strong><br>
 * Created: <strong>06.07.2019</strong><br>
 * Author: <strong>Kai S. K. Engelbart</strong>
 */
public class NaturalPlayer extends Player {

	private boolean moveRequested;

	public NaturalPlayer(Color color, OverlayComponent overlayComponent) {
		super(color);
		moveRequested = false;
		overlayComponent.addMouseListener(new MouseAdapter() {

			private Position pos;

			@Override
			public void mousePressed(MouseEvent evt) {
				if (!moveRequested) return;
				if (pos == null) {
					pos = new Position(evt.getPoint().x / overlayComponent.getTileSize(),
							evt.getPoint().y / overlayComponent.getTileSize());

					Board board = (Board) NaturalPlayer.this.board.clone();
					if (board.get(pos) != null && board.get(pos).getColor() == color) {
						List<Position> positions = board.getMoves(pos)
							.stream()
							.map(move -> move.dest)
							.collect(Collectors.toList());
						overlayComponent.displayDots(positions);
					} else pos = null;
				} else {
					Position dest = new Position(evt.getPoint().x / overlayComponent.getTileSize(),
							evt.getPoint().y / overlayComponent.getTileSize());

					overlayComponent.clearDots();
					moveRequested = false;
					game.onMove(NaturalPlayer.this, new Move(pos, dest, Type.UNKNOWN));
					pos = null;
				}
			}
		});
	}

	@Override
	public void requestMove() {
		moveRequested = true;
	}

	@Override
	public void cancelMove() {
		moveRequested = false;
	}
}
